package futils.fonts;
import java.awt.*;
import java.applet.Applet;

//////////////////////////////////////////////////////////////////////////////

public class FontViewer extends Applet
  {
  	static Button doneButton = new Button("done");
  	


public boolean handleEvent(Event evt) {

	if (evt.id == evt.ACTION_EVENT && evt.target == doneButton) {
		getParent().hide();
		return true;
	}	
	if (evt.id == evt.WINDOW_DESTROY ) {
		hide();
		return true;
	}	
	return super.handleEvent(evt); 
}
  public void init() {
    addNotify();
    Insets insets = insets();
    width=insets.right - insets.left;
    height=100;
    resize(width,height);

    Panel panel=new Panel();
    canvas=new FontCanvas();
    nameField=new FontNameField(canvas);
    baseField=new BaseField(canvas);

    add(panel);
    panel.add(new Label("Font:"));
    panel.add(nameField);
    panel.add(new Label("     Format:"));
    panel.add(baseField);
    panel.add(doneButton);

    add(canvas);
    }

  public void start() {
    nameField.select(dfltFont);
    canvas.setFont(dfltFont);
    baseField.select("Octal");
    canvas.setBase(8);
    
    }

  private final String  dfltFont="TimesRoman";
  private  int     width;
  private  int     height;
  private FontCanvas    canvas;
  private FontNameField nameField;
  private BaseField     baseField;
  };

//////////////////////////////////////////////////////////////////////////////

class FontNameField extends Choice {
  public FontNameField(FontCanvas fontCanvas_)  {
    fontCanvas=fontCanvas_;
    String fonts[]=Toolkit.getDefaultToolkit().getFontList();
    for(int index=0; index<fonts.length; index++)
      addItem(fonts[index]);
    }

  public boolean action(Event event, Object what) {
    if(event.id==Event.ACTION_EVENT)   {
      fontCanvas.setFont(getSelectedItem());
      return true;
      }
    else
      return super.action(event,what);
    }

  private FontCanvas fontCanvas;
  }

//////////////////////////////////////////////////////////////////////////////

class BaseField extends Choice {
  public BaseField(FontCanvas fontCanvas_) {
    fontCanvas=fontCanvas_;
    addItem("Octal");
    addItem("Decimal");
    addItem("Hexadecimal");
    }

  public boolean action(Event event, Object what) {
    if(event.id==Event.ACTION_EVENT)  {
      if(getSelectedItem()=="Octal")
        fontCanvas.setBase(8);
      else if(getSelectedItem()=="Decimal")
        fontCanvas.setBase(10);
      else if(getSelectedItem()=="Hexadecimal")
        fontCanvas.setBase(16);
      return true;
      }
    else
      return super.action(event,what);
    }

  private FontCanvas fontCanvas;
  }

//////////////////////////////////////////////////////////////////////////////

class FontCanvas extends Canvas {
  public void setFont(String fontName_)  {
    fontName=fontName_;
        resize(width,height);
    repaint();
    }

  public void setBase(int base_) {
    base=base_;
    repaint();
    }

  public synchronized void update(Graphics g) {
    
    width = size().width;
    height = size().height;

    paint(g);

    }

  public void paint(Graphics g)  {

	Font numberFont = new Font("Courier", Font.PLAIN, pointSize);
	g.setFont(new Font(fontName, Font.PLAIN,pointSize));
    g.setColor(Color.white);
    g.fillRect(0,0,width,height);
       String big_number = "377";
       String tab_string ="M";
       String big_string = "M"+tab_string;

    int colWid = getFontMetrics(
		   		g.getFont()).stringWidth(big_string);
     colWid = colWid + getFontMetrics(
		   		numberFont).stringWidth(big_number);
	int leading = getFontMetrics(
		   		g.getFont()).getHeight();
	int tabWid =  getFontMetrics(
		   		g.getFont()).stringWidth(tab_string);
		   		
	 int x=leftBorder, y=leading;

    for(int index=0; index<256; index++) {
      g.setColor(Color.black);
      g.drawString(new Character((char)index).toString(),x,y);
      g.setColor(Color.gray);
      g.setFont(numberFont);
      g.drawString(Integer.toString(index,base),x+tabWid,y);
      g.setFont(new Font(fontName, Font.PLAIN,pointSize));
      if(y+leading>height-200)
        {
        x+=colWid;
        y=leading;
        }
      else
        y+=leading;
      }
    }

  private final int pointSize=12;
 int width=600;
  int height=550;
  private final int leftBorder=30;
  private int       base=10;
  private String    fontName;
  };
