package net;

import java.applet.*;
import java.awt.*;
import java.net.*;
import java.applet.*;
import java.io.*;
import java.util.Date;
import gui.ClosableFrame;

/**
 * The applet.
 */

public class CheckMail extends Applet
{
	String serverHostName = null;
	boolean standalone = false;
	int popPort = 110;
	Socket socket;
	PrintStream ps;
	DataInputStream dis;
	InetAddress rina;
	InetAddress lina;

	CheckMailForm form;

	/**
	 * Initialize the applet.
	 */

	public void init()
	{
		form = new CheckMailForm(this);
		add(form);
		form.resize(300, 300);
		setBackground(Color.blue);
		if (serverHostName == null) serverHostName = getCodeBase().getHost();
	}

	/**
	 * Show status text to the user.
	 */

	public void showStatus(String s)
	{
		System.out.println(s);
		if (standalone) return;
		super.showStatus(s);
	}

	/**
	 * Perform check for e-mail.
	 */

	public void checkForMail()
		throws IOException, NumberFormatException, Exception
	{
		showStatus("Checking for mail on " + serverHostName);

		// Open connection
		socket = new Socket(serverHostName, popPort);

		String rs;
		try
		{
			rina = socket.getInetAddress();
			lina = rina.getLocalHost();
			ps = new PrintStream(socket.getOutputStream());
			dis = new DataInputStream(socket.getInputStream());

			// Check for messages
			sendline("USER " + form.getId());
			rs = sendline("PASS " + form.getPswd());
			if (rs.charAt(0) != '+') throw new Exception("Incorrect password");
			rs = sendline("STAT");
		}
		catch (Exception ex)
		{
			socket.close();
			throw ex;
		}

		// Close connection
		socket.close();

		// Parse result
		int r = 0;
		r = Integer.parseInt(rs.substring(4, rs.indexOf(" messages")));

		// Update result field
		form.resultField.setText(Integer.toString(r));
	}

	/**
	 * Send a line of data to the server, and return the handshake.
	 */

	String sendline(String data)
		throws IOException
	{
		System.out.println("sendline out:" + data);
		ps.println(data);
		ps.flush();
		String s = dis.readLine();
		System.out.println("sendline in:" + s);
		return s;
	}

	/**
	 * Main routine, for running as a standalone application.
	 */

	public static void main(String args[])
	{
		CheckMail ap = new CheckMail();
		ap.serverHostName = "mail.snet.net";
		ap.standalone = true;

		ClosableFrame fr = new ClosableFrame("CheckMail");
		ap.init();
		fr.add("Center", ap);
		fr.resize(300, 300);

		fr.show();
		ap.start();
	}

}

/**
 * Form for obtaining e-mail user id and password from the user.
 */

class CheckMailForm extends Panel
{
	CheckMail applet;

	// The form's elements...
	Label idLabel;
	TextField idField;
	Label pswdLabel;
	TextField pswdField;
	Button button;
	Label resultLabel;
	TextField resultField;

	/**
	 * The constructor.
	 */

	public CheckMailForm(CheckMail ap)
	{
		applet = ap;
		setBackground(Color.blue);
		setLayout(new GridLayout(7, 1));

		// Instantiate all the elements, and add them to the form...
		add(button = new Button("Check For Mail"));
		add(idLabel = new Label("Id:"));
		add(idField = new TextField(20));
		add(pswdLabel = new Label("Password:"));
		add(pswdField = new TextField(20));
		pswdField.setEchoCharacter('*');
		add(resultLabel = new Label("Messages Waiting:"));
		add(resultField = new TextField(6));
		resultField.setEditable(false);

		// Set the size of the form
		resize(250, 250);
	}

	/**
	 * Return the value of the ID field in the form.
	 */

	public String getId()
	{
		return idField.getText();
	}

	/**
	 * Return the value of the password field in the form.
	 */

	public String getPswd()
	{
		return pswdField.getText();
	}

	/**
	 * Respond to the button click event: check for messages.
	 */

	public boolean handleEvent(Event e)
	{
		if ((e.target == button) && (e.id == Event.ACTION_EVENT))
		{
			try  {applet.checkForMail();}
			catch (Exception ex)
			{
				applet.showStatus("Error; unable to check for messages:\n  "
					 + ex.toString());
				return true;
			}
			applet.showStatus("Completed.");

			return true;
		}

		return super.handleEvent(e);
	}
}

