package observers;
import java.awt.*;
import java.util.*;
//
// IntScrollbar - a Scrollbar that modifies an
// ObservableInt.  This class functions as both a
// "view" of the observable, since the position of
// the scrollbar is changed as the observable's value
// is changed, and it is a "controller" since it also
// sets the value of the observable.
//
// IntScrollbar has the same constructors as Scrollbar,
// except that in each case, there is an additional
// parameter that is the ObservableInt.
// Note:  On the constructor where you pass in the initial
// scrollbar position, the position is ignored.
public class IntScrollbar extends Scrollbar implements Observer
{
     private ObservableInt intValue;
// The bulk of this class is implementing the various
// constructors that are available in the Scrollbar class.
     public IntScrollbar(ObservableInt newValue)
     {
          super();     // Call the Scrollbar constructor
          intValue = newValue;
          intValue.addObserver(this);        // Register interest
          setValue(intValue.getValue());     // Change scrollbar position
     }
     public IntScrollbar(ObservableInt newValue, int orientation)
     {
          super(orientation);         // Call the Scrollbar constructor
          intValue = newValue;
          intValue.addObserver(this);        // Register interest
          setValue(intValue.getValue());     // Change scrollbar position
     }
     public IntScrollbar(ObservableInt newValue, int orientation,
          int value, int pageSize, int lowValue, int highValue)
     {
          super(orientation, value, pageSize, lowValue, highValue);
          intValue = newValue;
          intValue.addObserver(this);        // Register interest
          setValue(intValue.getValue());     // Change scrollbar position
     }
// The handleEvent method check with the parent class (Scrollbar) to see
// if it wants the event, if not, just assume the scrollbar value has
// changed and update the observable int with the new position.
     public boolean handleEvent(Event evt)
     {
          if (super.handleEvent(evt))
          {
               return true;     // The Scrollbar class handled it

          }
          intValue.setValue(getValue());     // Update the observable int
          return true;
     }
// update is called whenever the observable int changes its value
     public void update(Observable obs, Object arg)
     {
          setValue(intValue.getValue());
     }
}